unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, ExtCtrls, StdCtrls, Grids, Spin, Types, Math;

type

  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    dg: TDrawGrid;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Panel1: TPanel;
    sebs: TSpinEdit;
    sebc: TSpinEdit;
    procedure Button1Click(Sender: TObject);
    procedure dgDrawCell(Sender: TObject; aCol, aRow: Integer; aRect: TRect; aState: TGridDrawState);
    procedure dgMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure dgMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    procedure DrawMap;
  public

  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

const
  colorarr: array [0..4] of TColor = (clWhite, clYellow, $008080FF, clAqua, clLime);

var
  maparr: array of array of String;
  mapx, blocksize, blockspace: Integer;

procedure TForm1.Button1Click(Sender: TObject);
var
  i, r: Integer;
begin
  case sebs.Value of
    1..7: blockspace := 1;
    8..15: blockspace := 2;
    16..32: blockspace := 4
    else
      blockspace := 8;
  end;
  blocksize := sebs.Value + blockspace;
  SetLength(maparr, sebc.Value, 2);
  for i := Low(maparr) to High(maparr) do
  begin
    r := Random(Length(colorarr));
    maparr[i, 0] := 'Hint for cell ' + IntToStr(i) + '; Color: ' + ColorToString(colorarr[r]);
    maparr[i, 1] := IntToStr(r);
  end;
  DrawMap;
end;

procedure TForm1.dgDrawCell(Sender: TObject; aCol, aRow: Integer; aRect: TRect; aState: TGridDrawState);
begin
  if (aRow * mapx) + aCol < Length(maparr) then
  begin
    dg.Canvas.Brush.Color := colorarr[StrToInt(maparr[(aRow * mapx) + aCol, 1])];
    dg.Canvas.FillRect(aRect);
  end;
end;

procedure TForm1.dgMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  aCol, aRow, i, r: Integer;
begin
  if (dg.ColCount > 0) and (dg.RowCount > 0) then
  begin
    dg.MouseToCell(X, Y, aCol, aRow);
    i := (aRow * mapx) + aCol;
    if i < Length(maparr) then
    begin
      r := StrToInt(maparr[i, 1]) + 1;
      if r > High(colorarr) then
        r := Low(colorarr);
      maparr[i, 0] := 'Hint for cell ' + IntToStr(i) + '; Color: ' + ColorToString(colorarr[r]);
      maparr[i, 1] := IntToStr(r);
    end;
  end;
end;

procedure TForm1.dgMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
  aCol, aRow: Integer;
begin
  if (dg.ColCount > 0) and (dg.RowCount > 0) then
  begin
    dg.MouseToCell(X, Y, aCol, aRow);
    if (aRow * mapx) + aCol < Length(maparr) then
      dg.Hint := maparr[(aRow * mapx) + aCol, 0]
    else
      dg.Hint := '';
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Randomize;
  mapx := 0;
  dg.Clear;
  dg.FixedCols := 0;
  dg.FixedRows := 0;
  dg.Color := clBlack;
  dg.FocusRectVisible := False;
  dg.GridLineColor := dg.Color;
  dg.ShowHint := True;
  blockspace := 2;
  blocksize := sebs.Value + blockspace;
  Application.HintPause := 10;
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  DrawMap;
end;

procedure TForm1.DrawMap;
begin
  dg.BeginUpdate;
  dg.Clear;
  dg.GridLineWidth := blockspace;
  dg.DefaultColWidth := blocksize;
  dg.DefaultRowHeight := blocksize;
  mapx := dg.ClientWidth div blocksize;
  dg.ColCount := mapx;
  dg.RowCount := Ceil(Length(maparr) / mapx);
  dg.EndUpdate;
  if (Ceil(Length(maparr) / mapx) * blocksize) > dg.ClientHeight then
  begin
    dg.BeginUpdate;
    mapx := dg.ClientWidth div blocksize;
    dg.ColCount := mapx;
    dg.RowCount := Ceil(Length(maparr) / mapx);
    dg.EndUpdate;
  end;
end;

end.
